(define-module (ui mastercontrol ajax)
  :use-module (alterator ajax)
  :use-module (alterator ahttpd)
  :use-module (alterator plist)
  :use-module (alterator woo)
  :export (init))

(define *limit* 25)
(define *skip* 0)

(define (read-sources)
  (catch/message
    (lambda ()
      (let ((sources (woo-list "/mastercontrol/sources")))
        (form-update-enum "source" sources)
        (let ((source (ahttpd-session-ref 'mastercontrol-source)))
          (form-update-value "source" (if (and source (not (string-null? source)))
                                        source
                                        (and (not (null? sources))
                                             (cdr (plistq 'name (cdar sources)))))))))))

(define (update-filter)
  (catch/message
    (lambda ()
      (let* ((filter-data (woo-read-first (string-append "/mastercontrol/filter/" (form-value "source"))
                                          'filter (ahttpd-session-ref 'mastercontrol-filterexpr)))
             (sql (woo-get-option filter-data 'sql))
             (desc (woo-get-option filter-data 'desc)))
        (ahttpd-session-set! 'mastercontrol-filtersql sql)
        (if (and desc (not (string-null? desc)))
          (form-update-value "filter" desc)
          (form-update-value "filter" (_ "empty")))))))

(define (read-groups)
  (catch/message
    (lambda ()
      (let ((groups (woo-list (string-append "/mastercontrol/groups/" (form-value "source")))))
        (form-update-enum "group" groups)
        (if (not (null? groups))
          (begin
            (form-update-value "group" (cdr (plistq 'name (cdar groups))))
            (form-update-visibility "groups" #t))
          (form-update-visibility "groups" #f))))))

(define (format-row row proc)
  (if (plist? row)
    (apply proc (cdr (plist-fold
                       (lambda (n v ns-vs)
                         (cons
                           (append (car ns-vs) (list n))
                           (append (cdr ns-vs) (list v))))
                       (cons '() '())
                       row)))
    (cons (car row) (format-row (cdr row) proc))))

(define (format-host source id name address profile-hash current)
  (let* ((current-list (string-split current #\tab))
         (current-profile-hash (and (> (length current-list) 0) (car current-list)))
         (current-profile-name (and (> (length current-list) 1) (cadr current-list)))
         (current-profile-message (and (> (length current-list) 2) (caddr current-list))))
    (list 'name (string-append source "/" id)
          'hostname name
          'address address
          'profile-name (if (and (not (equal? profile-hash "NOTSET")) current-profile-name)
                          current-profile-name
                          "")
          'profile-message (if (not (or (equal? profile-hash "NOTSET")
                                        (equal? profile-hash "UNKNOWN")))
                             (if (and current-profile-hash
                                      (not (equal? current-profile-hash "UNKNOWN")))
                               (or current-profile-message "")
                               (_ "no data"))
                             (_ "profile is not set"))
          'profile-hash (if (not (or (equal? profile-hash "NOTSET")
                                     (equal? profile-hash "UNKNOWN")
                                     (or (not current-profile-hash)
                                         (equal? current-profile-hash "UNKNOWN"))))
                          current-profile-hash
                          "")
          'expected-hash (if (not (or (equal? profile-hash "NOTSET")
                                      (equal? profile-hash "UNKNOWN")))
                           profile-hash
                           "")
          'status (cond
                    ((equal? current "DISABLED")
                     (_ "not monitored"))
                    ((equal? current "UNKNOWN")
                     (_ "no data"))
                    ((and current-profile-hash
                          (equal? profile-hash current-profile-hash))
                     (_ "OK"))
                    (else (_ "configuration changed")))
          'class (cond
                   ((equal? current "DISABLED")
                    "")
                   ((equal? current "UNKNOWN")
                    "")
                   ((and current-profile-hash
                         (equal? profile-hash current-profile-hash))
                    "A")
                   (else "M")))))

(define (read-stats)
  (catch/message
    (lambda ()
      (form-update-value-list '("total" "disabled" "unknown" "modified")
                              (woo-read-first (string-append "/mastercontrol/hosts/stats/" (form-value "source"))
                                              'host (form-value "host")
                                              'hoststatus (form-value "hoststatus")
                                              'group (form-value "group")
                                              'filtersql (ahttpd-session-ref 'mastercontrol-filtersql))))))

(define (marked-hosts)
  (filter (lambda (a) (not (string-null? a)))
          (string-split (or (form-value "hosts") "") #\;)))

(define (update-preferred-range)
  (form-bind "hosts" "change"
    (lambda ()
      (if (not (null? (marked-hosts)))
        (form-update-value "update-range" "marked")
        (form-update-value "update-range" "all")))))

(define (read-hosts)
  (read-stats)
  (catch/message
    (lambda ()
      (let* ((hosts (woo-list (string-append "/mastercontrol/hosts/" (form-value "source"))
                              'host (form-value "host")
                              'hoststatus (form-value "hoststatus")
                              'group (form-value "group")
                              'filtersql (ahttpd-session-ref 'mastercontrol-filtersql)
                              'limit (+ *limit* 1)
                              'skip *skip*)))
        (if (> (length hosts) *limit*)
          (begin
            (form-update-enum "hosts"
                              (map (lambda (row)
                                     (format-row row
                                                 (lambda (id name address profile current)
                                                   (format-host (form-value "source") id name address profile current))))
                                   (list-head hosts *limit*)))
            (update-preferred-range)
            #t)
          (begin
            (form-update-enum "hosts"
                              (map (lambda (row)
                                     (format-row row
                                                 (lambda (id name address profile current)
                                                   (format-host (form-value "source") id name address profile current))))
                                   hosts))
            (update-preferred-range)
            #f))))))

(define (mark-hosts hostlist)
  (if hostlist
    (begin
      (form-update-value "hosts" hostlist)
      (form-update-value "update-range" "marked"))
    (form-update-value "update-range" "all")))

(define (read-hosts-from-first)
  (set! *skip* 0)
  (form-update-visibility "prev" #f)
  (form-update-visibility "next" (read-hosts)))

(define (init)
  (form-bind "source" "change"
    (lambda ()
      (update-filter)
      (read-groups)
      (read-hosts-from-first)
      (ahttpd-session-set! 'mastercontrol-source (form-value "source"))))
  (form-bind "refresh" "click"
    (lambda ()
      (form-update-visibility "next" (read-hosts))))
  (form-bind "clearfilter" "click"
    (lambda ()
      (ahttpd-session-set! 'mastercontrol-filterexpr #f)
      (update-filter)
      (read-hosts-from-first)))
  (form-bind "clearhost" "click"
    (lambda ()
      (form-update-value "host" "")
      (ahttpd-session-set! 'mastercontrol-host #f)
      (read-hosts-from-first)))
  (form-bind "host" "enter"
    (lambda ()
      (read-hosts-from-first)
      (ahttpd-session-set! 'mastercontrol-host (form-value "host"))))
  (form-bind "group" "change"
    (lambda ()
      (read-hosts-from-first)
      (ahttpd-session-set! 'mastercontrol-hostgroup (form-value "group"))))
  (form-bind "prev" "click"
    (lambda ()
      (if (> *skip* 0)
        (set! *skip*
          (if (> *skip* *limit*)
            (- *skip* *limit*)
            0)))
      (form-update-visibility "prev" (> *skip* 0))
      (form-update-visibility "next" (read-hosts))))
  (form-bind "next" "click"
    (lambda ()
      (set! *skip* (+ *skip* *limit*))
      (form-update-visibility "prev" #t)
      (form-update-visibility "next" (read-hosts))))
  (form-bind "status-disabled" "click"
    (lambda ()
      (form-update-value "hoststatus" "disabled")
      (read-hosts-from-first)
      (ahttpd-session-set! 'mastercontrol-hoststatus "disabled")))
  (form-bind "status-notset" "click"
    (lambda ()
      (form-update-value "hoststatus" "notset")
      (read-hosts-from-first)
      (ahttpd-session-set! 'mastercontrol-hoststatus "notset")))
  (form-bind "status-modified" "click"
    (lambda ()
      (form-update-value "hoststatus" "modified")
      (read-hosts-from-first)
      (ahttpd-session-set! 'mastercontrol-hoststatus "modified")))
  (form-bind "hoststatus" "change"
    (lambda ()
      (read-hosts-from-first)
      (ahttpd-session-set! 'mastercontrol-hoststatus (form-value "hoststatus"))))
  (form-bind "dispatch" "click"
    (lambda ()
      (if (equal? (form-value "update-range") "all")
        (form-replace "/mastercontrol/dispatch" 'update-range "all")
        (form-replace "/mastercontrol/dispatch" 'update-range "marked"
                                                'hostlist (form-value "hosts")))))
  (read-sources)
  (update-filter)
  (read-groups)
  (form-update-visibility "prev" (> *skip* 0))
  (form-update-visibility "next" (read-hosts))
  (mark-hosts (form-value "hostlist"))
  (ahttpd-session-set! 'mastercontrol-source (form-value "source"))
  (ahttpd-session-set! 'mastercontrol-host (form-value "host"))
  (ahttpd-session-set! 'mastercontrol-hoststatus (form-value "hoststatus"))
  (ahttpd-session-set! 'mastercontrol-hostgroup (form-value "group")))
