(define-module (ui mastercontrol params ajax)
  :use-module (alterator ajax)
  :use-module (alterator ahttpd)
  :use-module (alterator plist)
  :use-module (alterator woo)
  :export (init))

(define (format-row row proc)
  (if (plist? row)
    (apply proc (cdr (plist-fold
                       (lambda (n v ns-vs)
                         (cons
                           (append (car ns-vs) (list n))
                           (append (cdr ns-vs) (list v))))
                       (cons '() '())
                       row)))
    (cons (car row) (format-row (cdr row) proc))))

(define (format-value val type)
  (case (string->symbol type)
    ((string)
     (if (> (string-length val) 16)
       (string-append (string-take val 16) "…")
       val))
    (else val)))

(define (format-param id type minval avgval maxval label)
  (list 'id id
        'type type
        'minval (format-value minval type)
        'avgval (format-value avgval type)
        'maxval (format-value maxval type)
        'label label))

(define (read-params source)
  (catch/message
    (lambda ()
      (form-update-enum "params" (map (lambda (row)
                                        (format-row row format-param))
                                      (woo-list (string-append "/mastercontrol/params/" source)))))))

(define (init)
  (form-bind "setfilter" "click"
    (lambda ()
      (ahttpd-session-set! 'mastercontrol-filterexpr (cdr (assoc 'filter (js-result 'getFilter))))
      (form-replace "/mastercontrol")))
  (form-update-value "source" (ahttpd-session-ref 'mastercontrol-source))
  (read-params (ahttpd-session-ref 'mastercontrol-source)))
